<?php

namespace Drupal\Tests\usfedgov_google_analytics\Functional;

use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;

/**
 * Page attachments test.
 *
 * @group usfedgov_google_analytics
 */
class PageAttachmentTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'usfedgov_google_analytics'];

  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = FALSE;

  /**
   * A user with permission to administer settings.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * The path to a node that is created for testing.
   *
   * @var string
   */
  protected $nodePath;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // We need to test various routes where the script should not be attached.
    // But the script is also not attached if a user is logged in. Thus, access
    // must be granted to the anonymous user for any page that needs to be
    // tested.
    Role::load('anonymous')
      ->grantPermission('access content')
      ->grantPermission('administer federal google analytics')
      ->save();

    $this->adminUser = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'administer federal google analytics',
    ]);

    // In the testing profile the front page is set to /user. So the DAP script
    // won't be attached to it. We must generate a page on which the script will
    // be attached.
    $this->drupalCreateContentType(['type' => 'page']);
    $this->nodePath = "node/" . $this->drupalCreateNode(['promote' => 1])->id();
  }

  /**
   * @covers usfedgov_google_analytics_page_attachments
   */
  public function testPageAttachments(): void {
    $config = \Drupal::configFactory()->getEditable('usfedgov_google_analytics.settings');
    $script_html = '<script src="https://dap.digitalgov.gov/Universal-Federated-Analytics-Min.js';

    // Ensure the script is attached for anonymous users on "normal" pages.
    $this->drupalGet($this->nodePath);
    $this->assertSession()->responseContains($script_html);

    // Ensure the script is not attached on /user pages.
    $this->drupalGet(Url::fromRoute('user.login'));
    $this->assertSession()->responseNotContains($script_html);

    // Ensure the script is not attached on admin routes.
    $this->drupalGet(Url::fromRoute('usfedgov_google_analytics.form'));
    // This check is here for feedback in GitLab CI where we can't check the
    // HTML output. It's helpful to know that access to the page was denied
    // instead of only getting a message that the script wasn't found.
    $this->assertSession()->pageTextNotContains('Access denied');
    $this->assertSession()->responseNotContains($script_html);

    // Ensure the script is not attached for logged-in users.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($this->nodePath);
    $this->assertSession()->responseNotContains($script_html);
    $this->drupalLogout();
  }

}
